@@ -14,11 +14,10 @@ import scipy.optimize as opt
 
 from .recspeed import local_maxima, remove_similar_vertices, search_descending
 from dipy.core.sphere import HemiSphere, Sphere
-from dipy.data import get_sphere
+from dipy.data import default_sphere
 from dipy.core.ndindex import ndindex
 from dipy.reconst.shm import sh_to_sf_matrix
-
-default_sphere = HemiSphere.from_sphere(get_sphere('symmetric724'))
+from .peak_direction_getter import PeaksAndMetricsDirectionGetter
 
 
 def peak_directions_nl(sphere_eval, relative_peak_threshold=.25,
@@ -157,14 +156,14 @@ def peak_directions(odf, sphere, relative_peak_threshold=.5,
     return directions, values, indices
 
 
-class PeaksAndMetrics(object):
+class PeaksAndMetrics(PeaksAndMetricsDirectionGetter):
     pass
 
 
 def _peaks_from_model_parallel(model, data, sphere, relative_peak_threshold,
                                min_separation_angle, mask, return_odf,
-                               return_sh, gfa_thr, normalize_peaks,
-                               sh_order, sh_basis_type, npeaks, B, invB, nbr_processes):
+                               return_sh, gfa_thr, normalize_peaks, sh_order,
+                               sh_basis_type, npeaks, B, invB, nbr_processes):
 
     if nbr_processes is None:
         try:
@@ -219,6 +218,7 @@ def _peaks_from_model_parallel(model, data, sphere, relative_peak_threshold,
         pool.close()
 
         pam = PeaksAndMetrics()
+        pam.sphere = sphere
         # use memmap to reduce the memory usage
         pam.gfa = np.memmap(path.join(tmpdir, 'gfa.npy'),
                             dtype=pam_res[0].gfa.dtype,
@@ -370,9 +370,11 @@ def peaks_from_model(model, data, sphere, relative_peak_threshold,
         Inverse of B.
     parallel: bool
         If True, use multiprocessing to compute peaks and metric
-        (default False).
+        (default False). Temporary files are saved in the default temporary
+        directory of the system. It can be changed using ``import tempfile``
+        and ``tempfile.tempdir = '/path/to/tempdir'``.
     nbr_processes: int
-        If `parallel == True`, the number of subprocesses to use
+        If `parallel` is True, the number of subprocesses to use
         (default multiprocessing.cpu_count()).
 
     Returns
@@ -469,6 +471,7 @@ def peaks_from_model(model, data, sphere, relative_peak_threshold,
     qa_array /= global_max
 
     pam = PeaksAndMetrics()
+    pam.sphere = sphere
     pam.peak_dirs = peak_dirs
     pam.peak_values = peak_values
     pam.peak_indices = peak_indices
