@@ -29,10 +29,22 @@ from numpy import concatenate, diag, diff, empty, eye, sqrt, unique, dot
 from numpy.linalg import pinv, svd
 from numpy.random import randint
 from dipy.reconst.odf import OdfModel, OdfFit
-from scipy.special import sph_harm, lpn
+from scipy.special import sph_harm, lpn, lpmv, gammaln
+from dipy.core.sphere import Sphere
+import dipy.core.gradients as grad
+from dipy.sims.voxel import single_tensor, all_tensor_evecs
 from dipy.core.geometry import cart2sphere
+from dipy.core.onetime import auto_attr
 from dipy.reconst.cache import Cache
 
+from distutils.version import StrictVersion
+import scipy
+
+if StrictVersion(scipy.version.short_version) >= StrictVersion('0.15.0'):
+    SCIPY_15_PLUS = True
+else:
+    SCIPY_15_PLUS = False
+
 
 def _copydoc(obj):
     def bandit(f):
@@ -41,10 +53,152 @@ def _copydoc(obj):
     return bandit
 
 
-def real_sph_harm(m, n, theta, phi):
+def forward_sdeconv_mat(r_rh, n):
+    """ Build forward spherical deconvolution matrix
+
+    Parameters
+    ----------
+    r_rh : ndarray
+        Rotational harmonics coefficients for the single fiber response
+        function. Each element `rh[i]` is associated with spherical harmonics
+        of degree `2*i`.
+    n : ndarray
+        The degree of spherical harmonic function associated with each row of
+        the deconvolution matrix. Only even degrees are allowed
+
+    Returns
+    -------
+    R : ndarray (N, N)
+        Deconvolution matrix with shape (N, N)
+
     """
-    Compute real spherical harmonics, where the real harmonic $Y^m_n$ is
-    defined to be:
+
+    if np.any(n % 2):
+        raise ValueError("n has odd degrees, expecting only even degrees")
+    return np.diag(r_rh[n // 2])
+
+def sh_to_rh(r_sh, m, n):
+    """ Spherical harmonics (SH) to rotational harmonics (RH)
+
+    Calculate the rotational harmonic decomposition up to
+    harmonic order `m`, degree `n` for an axially and antipodally
+    symmetric function. Note that all ``m != 0`` coefficients
+    will be ignored as axial symmetry is assumed. Hence, there
+    will be ``(sh_order/2 + 1)`` non-zero coefficients.
+
+    Parameters
+    ----------
+    r_sh : ndarray (N,)
+        ndarray of SH coefficients for the single fiber response function.
+        These coefficients must correspond to the real spherical harmonic
+        functions produced by `shm.real_sph_harm`.
+    m : ndarray (N,)
+        The order of the spherical harmonic function associated with each
+        coefficient.
+    n : ndarray (N,)
+        The degree of the spherical harmonic function associated with each
+        coefficient.
+
+    Returns
+    -------
+    r_rh : ndarray (``(sh_order + 1)*(sh_order + 2)/2``,)
+        Rotational harmonics coefficients representing the input `r_sh`
+
+    See Also
+    --------
+    shm.real_sph_harm, shm.real_sym_sh_basis
+
+    References
+    ----------
+    .. [1] Tournier, J.D., et al. NeuroImage 2007. Robust determination of the
+        fibre orientation distribution in diffusion MRI: Non-negativity
+        constrained super-resolved spherical deconvolution
+
+    """
+    mask = m == 0
+    # The delta function at theta = phi = 0 is known to have zero coefficients
+    # where m != 0, therefore we need only compute the coefficients at m=0.
+    dirac_sh = gen_dirac(0, n[mask], 0, 0)
+    r_rh = r_sh[mask] / dirac_sh
+    return r_rh
+
+
+def gen_dirac(m, n, theta, phi):
+    """ Generate Dirac delta function orientated in (theta, phi) on the sphere
+
+    The spherical harmonics (SH) representation of this Dirac is returned as
+    coefficients to spherical harmonic functions produced by
+    `shm.real_sph_harm`.
+
+    Parameters
+    ----------
+    m : ndarray (N,)
+        The order of the spherical harmonic function associated with each
+        coefficient.
+    n : ndarray (N,)
+        The degree of the spherical harmonic function associated with each
+        coefficient.
+    theta : float [0, 2*pi]
+        The azimuthal (longitudinal) coordinate.
+    phi : float [0, pi]
+        The polar (colatitudinal) coordinate.
+
+    See Also
+    --------
+    shm.real_sph_harm, shm.real_sym_sh_basis
+
+    Returns
+    -------
+    dirac : ndarray
+        SH coefficients representing the Dirac function. The shape of this is
+        `(m + 2) * (m + 1) / 2`.
+
+    """
+    return real_sph_harm(m, n, theta, phi)
+
+
+def spherical_harmonics(m, n, theta, phi):
+    r""" Compute spherical harmonics
+
+    This may take scalar or array arguments. The inputs will be broadcasted
+    against each other.
+
+    Parameters
+    ----------
+    m : int ``|m| <= n``
+        The order of the harmonic.
+    n : int ``>= 0``
+        The degree of the harmonic.
+    theta : float [0, 2*pi]
+        The azimuthal (longitudinal) coordinate.
+    phi : float [0, pi]
+        The polar (colatitudinal) coordinate.
+
+    Returns
+    -------
+    y_mn : complex float
+        The harmonic $Y^m_n$ sampled at `theta` and `phi`.
+
+    Notes
+    -----
+    This is a faster implementation of scipy.special.sph_harm for
+    scipy version < 0.15.0.
+
+    """
+    if SCIPY_15_PLUS:
+        return sph_harm(m, n, theta, phi)
+    x = np.cos(phi)
+    val = lpmv(m, n, x).astype(complex)
+    val *= np.sqrt((2 * n + 1) / 4.0 / np.pi)
+    val *= np.exp(0.5 * (gammaln(n - m + 1) - gammaln(n + m + 1)))
+    val = val * np.exp(1j * m * theta)
+    return val
+
+
+def real_sph_harm(m, n, theta, phi):
+    r""" Compute real spherical harmonics.
+
+    Where the real harmonic $Y^m_n$ is defined to be:
 
         Real($Y^m_n$) * sqrt(2) if m > 0
         $Y^m_n$                 if m == 0
@@ -75,13 +229,14 @@ def real_sph_harm(m, n, theta, phi):
     """
     # dipy uses a convention for theta and phi that is reversed with respect to
     # function signature of scipy.special.sph_harm
-    sh = sph_harm(np.abs(m), n, phi, theta)
+    sh = spherical_harmonics(np.abs(m), n, phi, theta)
+
     real_sh = np.where(m > 0, sh.imag, sh.real)
     real_sh *= np.where(m == 0, 1., np.sqrt(2))
     return real_sh
 
 
-def real_sph_harm_mrtrix(sh_order, theta, phi):
+def real_sym_sh_mrtrix(sh_order, theta, phi):
     """
     Compute real spherical harmonics as in mrtrix, where the real harmonic
     $Y^m_n$ is defined to be::
@@ -124,10 +279,13 @@ def real_sph_harm_mrtrix(sh_order, theta, phi):
     return real_sh, m, n
 
 
-def real_sph_harm_fibernav(sh_order, theta, phi):
-    """
-    Compute real spherical harmonics as in fibernavigator, where the real
-    harmonic $Y^m_n$ is defined to be::
+def real_sym_sh_basis(sh_order, theta, phi):
+    """Samples a real symmetric spherical harmonic basis at point on the sphere
+
+    Samples the basis functions up to order `sh_order` at points on the sphere
+    given by `theta` and `phi`. The basis functions are defined here the same
+    way as in fibernavigator [1]_ where the real harmonic $Y^m_n$ is defined to
+    be:
 
         Imag($Y^m_n$) * sqrt(2)     if m > 0
         $Y^m_n$                     if m == 0
@@ -148,8 +306,7 @@ def real_sph_harm_fibernav(sh_order, theta, phi):
     Returns
     --------
     y_mn : real float
-        The real harmonic $Y^m_n$ sampled at `theta` and `phi` as
-        implemented in the FiberNavigator [1]_.
+        The real harmonic $Y^m_n$ sampled at `theta` and `phi`
     m : array
         The order of the harmonics.
     n : array
@@ -168,17 +325,17 @@ def real_sph_harm_fibernav(sh_order, theta, phi):
     return real_sh, m, n
 
 
-sph_harm_lookup = {None: real_sph_harm_fibernav,
-                   "mrtrix": real_sph_harm_mrtrix,
-                   "fibernav": real_sph_harm_fibernav}
+sph_harm_lookup = {None: real_sym_sh_basis,
+                   "mrtrix": real_sym_sh_mrtrix,
+                   "fibernav": real_sym_sh_basis}
 
 
 def sph_harm_ind_list(sh_order):
     """
     Returns the degree (n) and order (m) of all the symmetric spherical
-    harmonics of degree less then or equal it sh_order. The results, m_list
-    and n_list are kx1 arrays, where k depends on sh_order. They can be
-    passed to real_sph_harm.
+    harmonics of degree less then or equal to `sh_order`. The results, `m_list`
+    and `n_list` are kx1 arrays, where k depends on sh_order. They can be
+    passed to :func:`real_sph_harm`.
 
     Parameters
     ----------
@@ -213,6 +370,15 @@ def sph_harm_ind_list(sh_order):
     return (m_list, n_list)
 
 
+def order_from_ncoef(ncoef):
+    """
+    Given a number n of coefficients, calculate back the sh_order
+    """
+    # Solve the quadratic equation derived from :
+    # ncoef = (sh_order + 2) * (sh_order + 1) / 2
+    return int(-3 + np.sqrt(9 - 4 * (2-2*ncoef)))/2
+
+
 def smooth_pinv(B, L):
     """Regularized psudo-inverse
 
@@ -261,10 +427,68 @@ def lazy_index(index):
         return slice(index[0], index[-1] + 1, step[0])
 
 
+def _gfa_sh(coef, sh0_index=0):
+    """The gfa of the odf, computed from the spherical harmonic coefficients
+
+    This is a private function because it only works for coefficients of
+    normalized sh bases.
+
+    Parameters
+    ----------
+    coef : array
+        The coefficients, using a normalized sh basis, that represent each odf.
+    sh0_index : int
+        The index of the coefficient associated with the 0th order sh harmonic.
+
+    Returns
+    -------
+    gfa_values : array
+        The gfa of each odf.
+
+    """
+    coef_sq = coef**2
+    numer = coef_sq[..., sh0_index]
+    denom = (coef_sq).sum(-1)
+    # The sum of the square of the coefficients being zero is the same as all
+    # the coefficients being zero
+    allzero = denom == 0
+    # By adding 1 to numer and denom where both and are 0, we prevent 0/0
+    numer = numer + allzero
+    denom = denom + allzero
+    return np.sqrt(1. - (numer / denom))
+
+
 class SphHarmModel(OdfModel, Cache):
-    """The base class to sub-classed by specific spherical harmonic models of
-    diffusion data"""
-    def __init__(self, gtab, sh_order, smooth=0, min_signal=1.,
+    """To be subclassed by all models that return a SphHarmFit when fit."""
+
+    def sampling_matrix(self, sphere):
+        """The matrix needed to sample ODFs from coefficients of the model.
+
+        Parameters
+        ----------
+        sphere : Sphere
+            Points used to sample ODF.
+
+        Returns
+        -------
+        sampling_matrix : array
+            The size of the matrix will be (N, M) where N is the number of
+            vertices on sphere and M is the number of coefficients needed by
+            the model.
+        """
+        sampling_matrix = self.cache_get("sampling_matrix", sphere)
+        if sampling_matrix is None:
+            sh_order = self.sh_order
+            theta = sphere.theta
+            phi = sphere.phi
+            sampling_matrix, m, n = real_sym_sh_basis(sh_order, theta, phi)
+            self.cache_set("sampling_matrix", sphere, sampling_matrix)
+        return sampling_matrix
+
+
+class QballBaseModel(SphHarmModel):
+    """To be subclassed by Qball type models."""
+    def __init__(self, gtab, sh_order, smooth=0.006, min_signal=1.,
                  assume_normed=False):
         """Creates a model that can be used to fit or sample diffusion data
 
@@ -274,23 +498,34 @@ class SphHarmModel(OdfModel, Cache):
             Diffusion gradients used to acquire data
         sh_order : even int >= 0
             the spherical harmonic order of the model
-        smoothness : float between 0 and 1
+        smooth : float between 0 and 1, optional
             The regularization parameter of the model
-        assume_normed : bool
-            If True, data will not be normalized before fitting to the model
+        min_signal : float, > 0, optional
+            During fitting, all signal values less than `min_signal` are
+            clipped to `min_signal`. This is done primarily to avoid values
+            less than or equal to zero when taking logs.
+        assume_normed : bool, optional
+            If True, clipping and normalization of the data with respect to the
+            mean B0 signal are skipped during mode fitting. This is an advanced
+            feature and should be used with care.
+
+        See Also
+        --------
+        normalize_data
 
         """
-        m, n = sph_harm_ind_list(sh_order)
+        SphHarmModel.__init__(self, gtab)
         self._where_b0s = lazy_index(gtab.b0s_mask)
         self._where_dwi = lazy_index(~gtab.b0s_mask)
         self.assume_normed = assume_normed
         self.min_signal = min_signal
         x, y, z = gtab.gradients[self._where_dwi].T
         r, theta, phi = cart2sphere(x, y, z)
-        B = real_sph_harm(m, n, theta[:, None], phi[:, None])
+        B, m, n = real_sym_sh_basis(sh_order, theta[:, None], phi[:, None])
         L = -n * (n + 1)
         legendre0 = lpn(sh_order, 0)[0]
         F = legendre0[n]
+        self.sh_order = sh_order
         self.B = B
         self.m = m
         self.n = n
@@ -361,17 +596,43 @@ class SphHarmFit(OdfFit):
             The value of the odf on each point of `sphere`.
 
         """
-        sampling_matrix = self.model.cache_get("sampling_matrix", sphere)
-        if sampling_matrix is None:
-            phi = sphere.phi.reshape((-1, 1))
-            theta = sphere.theta.reshape((-1, 1))
-            sampling_matrix = real_sph_harm(self.model.m, self.model.n,
-                                            theta, phi)
-            self.model.cache_set("sampling_matrix", sphere, sampling_matrix)
-        return dot(self._shm_coef, sampling_matrix.T)
+        B = self.model.sampling_matrix(sphere)
+        return dot(self._shm_coef, B.T)
+
+    @auto_attr
+    def gfa(self):
+        return _gfa_sh(self._shm_coef, 0)
+
+    @property
+    def shm_coeff(self):
+        """The spherical harmonic coefficients of the odf
+
+        Make this a property for now, if there is a usecase for modifying
+        the coefficients we can add a setter or expose the coefficients more
+        directly
+        """
+        return self._shm_coef
+
+    def predict(self, gtab=None, S0=1.0):
+        """
+        Predict the diffusion signal from the model coefficients.
+
+        Parameters
+        ----------
+        gtab : a GradientTable class instance
+            The directions and bvalues on which prediction is desired
+
+        S0 : float array
+           The mean non-diffusion-weighted signal in each voxel.
+           Default: 1.0 in all voxels
+        """
+        if not hasattr(self.model, 'predict'):
+            msg = "This model does not have prediction implemented yet"
+            raise NotImplementedError(msg)
+        return self.model.predict(self.shm_coeff, gtab, S0)
 
 
-class CsaOdfModel(SphHarmModel):
+class CsaOdfModel(QballBaseModel):
     """Implementation of Constant Solid Angle reconstruction method.
 
     References
@@ -381,6 +642,7 @@ class CsaOdfModel(SphHarmModel):
     """
     min = .001
     max = .999
+    _n0_const = .5 / np.sqrt(np.pi)
 
     def _set_fit_matrix(self, B, L, F, smooth):
         """The fit matrix, is used by fit_coefficients to return the
@@ -388,17 +650,19 @@ class CsaOdfModel(SphHarmModel):
         invB = smooth_pinv(B, sqrt(smooth) * L)
         L = L[:, None]
         F = F[:, None]
-        self._fit_matrix = F * L * invB
+        self._fit_matrix = (F * L) / (8 * np.pi) * invB
 
     def _get_shm_coef(self, data, mask=None):
         """Returns the coefficients of the model"""
         data = data[..., self._where_dwi]
         data = data.clip(self.min, self.max)
         loglog_data = np.log(-np.log(data))
-        return dot(loglog_data, self._fit_matrix.T)
+        sh_coef = dot(loglog_data, self._fit_matrix.T)
+        sh_coef[..., 0] = self._n0_const
+        return sh_coef
 
 
-class OpdtModel(SphHarmModel):
+class OpdtModel(QballBaseModel):
     """Implementation of Orientation Probability Density Transform
     reconstruction method.
 
@@ -431,7 +695,7 @@ def _slowadc_formula(data, delta_b, delta_q):
     return dot(logd * (1.5 - logd) * data, delta_q.T) - dot(data, delta_b.T)
 
 
-class QballModel(SphHarmModel):
+class QballModel(QballBaseModel):
     """Implementation of regularized Qball reconstruction method.
 
     References
@@ -496,7 +760,7 @@ def bootstrap_data_array(data, H, R, permute=None):
 
     data must be normalized, ie 0 < data <= 1
 
-    This function, and the bootstrap_data_voxel function, calculat
+    This function, and the bootstrap_data_voxel function, calculate
     residual-bootsrap samples given a Hat matrix and a Residual matrix. These
     samples can be used for non-parametric statistics or for bootstrap
     probabilistic tractography:
@@ -556,7 +820,7 @@ class ResidualBootstrapWrapper(object):
         signal_object : some object that can be indexed
             This object should return diffusion weighted signals when indexed.
         B : ndarray, ndim=2
-            The design matrix of spherical hormonic model usded to fit the
+            The design matrix of the spherical harmonics model used to fit the
             data. This is the model that will be used to compute the residuals
             and sample the residual bootstrap distribution
         where_dwi :
@@ -654,3 +918,47 @@ def sh_to_sf(sh, sphere, sh_order, basis_type=None):
     sf = np.dot(sh, B.T)
 
     return sf
+
+
+def sh_to_sf_matrix(sphere, sh_order, basis_type=None, return_inv=True, smooth=0):
+    """ Matrix that transforms Spherical harmonics (SH) to spherical
+    function (SF).
+
+    Parameters
+    ----------
+    sphere : Sphere
+        The points on which to sample the spherical function.
+    sh_order : int, optional
+        Maximum SH order in the SH fit.  For `sh_order`, there will be
+        ``(sh_order + 1) * (sh_order_2) / 2`` SH coefficients (default 4).
+    basis_type : {None, 'mrtrix', 'fibernav'}
+        ``None`` for the default dipy basis,
+        ``mrtrix`` for the MRtrix basis, and
+        ``fibernav`` for the FiberNavigator basis
+        (default ``None``).
+    return_inv : bool
+        If True then the inverse of the matrix is also returned
+    smooth : float, optional
+        Lambda-regularization in the SH fit (default 0.0).
+
+    Returns
+    -------
+    B : ndarray
+        Matrix that transforms spherical harmonics to spherical function
+        ``sf = np.dot(sh, B)``.
+    invB : ndarray
+        Inverse of B.
+
+    """
+    sph_harm_basis = sph_harm_lookup.get(basis_type)
+
+    if sph_harm_basis is None:
+        raise ValueError("Invalid basis name.")
+    B, m, n = sph_harm_basis(sh_order, sphere.theta, sphere.phi)
+
+    if return_inv:
+        L = -n * (n + 1)
+        invB = smooth_pinv(B, np.sqrt(smooth) * L)
+        return B.T, invB.T
+
+    return B.T
