@@ -5,12 +5,11 @@ from ...utils.six.moves import xrange
 import numpy as np
 import nose
 from dipy.io.bvectxt import orientation_from_string
-from dipy.tracking._utils import _rmi
 from dipy.tracking.utils import (affine_for_trackvis, connectivity_matrix,
                                  density_map, length, move_streamlines,
                                  ndbincount, reduce_labels,
                                  reorder_voxels_affine, seeds_from_mask,
-                                 target)
+                                 target, _rmi, unique_rows)
 
 import dipy.tracking.metrics as metrix 
 
@@ -410,3 +409,43 @@ def test_seeds_from_mask():
     in_444 = ((seeds > 3.5) & (seeds < 4.5)).all(1)
     assert_equal(in_444.sum(), 3 * 4 * 5)
 
+
+def test_connectivity_matrix_shape():
+    
+    # Labels: z-planes have labels 0,1,2
+    labels = np.zeros((3, 3, 3), dtype=int)
+    labels[:, :, 1] = 1
+    labels[:, :, 2] = 2  
+    # Streamline set, only moves between first two z-planes.
+    streamlines = [np.array([[0., 0., 0.],
+                             [0., 0., 0.5],
+                             [0., 0., 1.]]),
+                   np.array([[0., 1., 1.],
+                             [0., 1., 0.5],
+                             [0., 1., 0.]])]
+    matrix = connectivity_matrix(streamlines, labels, affine=np.eye(4))
+    assert_equal(matrix.shape, (3, 3))
+
+
+def test_unique_rows():
+    """
+    Testing the function unique_coords
+    """
+    arr = np.array([[1,2,3],[1,2,3],[2,3,4],[3,4,5]])
+    arr_w_unique = np.array([[1,2,3],[2,3,4],[3,4,5]])
+    assert_array_equal(unique_rows(arr), arr_w_unique)
+
+    # Should preserve order:
+    arr = np.array([[2,3,4],[1,2,3],[1,2,3],[3,4,5]])
+    arr_w_unique = np.array([[2,3,4],[1,2,3],[3,4,5]])
+    assert_array_equal(unique_rows(arr), arr_w_unique)
+
+
+    # Should work even with longer arrays:
+    arr = np.array([[2,3,4],[1,2,3],[1,2,3],[3,4,5],
+                    [6,7,8],[0,1,0],[1,0,1]])
+    arr_w_unique = np.array([[2,3,4],[1,2,3],[3,4,5],
+                             [6,7,8],[0,1,0],[1,0,1]])
+
+    assert_array_equal(unique_rows(arr), arr_w_unique)
+
